@@ -21,9 +21,9 @@
 
 #include "config.h"
 
-#include "bufrex_opcode.h"
-#include "bufrex_raw.h"
-#include <dballe/io/dba_rawfile.h>
+#include "opcode.h"
+#include "msg.h"
+#include <dballe/core/file.h>
 
 #include <stdio.h>
 #include <netinet/in.h>
@@ -38,7 +38,7 @@
 
 #include <assert.h>
 
-/* #define TRACE_DECODER */
+/*#define TRACE_DECODER*/
 
 #ifdef TRACE_DECODER
 #define TRACE(...) fprintf(stderr, __VA_ARGS__)
@@ -48,12 +48,25 @@
 #define IFTRACE if (0)
 #endif
 
+// Unmarshal a big endian integer value n bytes long
+static inline int readNumber(const unsigned char* buf, int bytes)
+{
+	int res = 0;
+	int i;
+	for (i = 0; i < bytes; ++i)
+	{
+		res <<= 8;
+		res |= buf[i];
+	}
+	return res;
+}
+
 struct _decoder 
 {
 	/* Input message data */
 	dba_rawmsg in;
 	/* Output decoded variables */
-	bufrex_raw out;
+	bufrex_msg out;
 
 	/* Offset of the start of BUFR section 1 */
 	const unsigned char* sec1;
@@ -66,9 +79,15 @@ struct _decoder
 	/* Offset of the start of BUFR section 5 */
 	const unsigned char* sec5;
 
+	/* 1 if the BUFR message uses compression, else 0 */
+	int compression;
+
 	/* Number of subsets present in the message */
 	int subsets;
 
+	/* Current subset when decoding non-compressed BUFR messages */
+	bufrex_subset current_subset;
+
 	/* Current value of scale change from C modifier */
 	int c_scale_change;
 	/* Current value of width change from C modifier */
@@ -108,8 +127,10 @@ static int decoder_bits_left(decoder d)
 	return (d->in->len - d->cursor) * 8 + d->pbyte_len;
 }
 
+#define FILENAME(d) ((d)->in->file == NULL ? "(memory)" : dba_file_name((d)->in->file))
+
 #define PARSE_ERROR(pos, ...) do { \
-		err = dba_error_parse(d->in->file->name, d->in->offset + (pos - d->in->buf), __VA_ARGS__); \
+		err = dba_error_parse(FILENAME(d), d->in->offset + ((pos) - d->in->buf), __VA_ARGS__); \
 		goto fail; \
 	} while (0)
 
@@ -148,33 +169,29 @@ fail:
 	return err;
 }
 
-#if 0
 /* Dump 'count' bits of 'buf', starting at the 'ofs-th' bit */
-static dba_err dump_bits(void* buf, int ofs, int count, FILE* out)
+static void dump_next_bits(decoder d, int count, FILE* out)
 {
-	bitvec vec;
-	int i, j;
-	DBA_RUN_OR_RETURN(bitvec_create(&vec, "mem", 0, buf, (count + ofs) / 8 + 2));
-	for (i = 0, j = 0; i < ofs; i++, j++)
+	int cursor = d->cursor;
+	int pbyte = d->pbyte;
+	int pbyte_len = d->pbyte_len;
+	int i;
+
+	for (i = 0; i < count; ++i) 
 	{
-		uint32_t val;
-		DBA_RUN_OR_RETURN(bitvec_get_bits(vec, 1, &val));
-		if (j != 0 && (j % 8) == 0)
+		if (d->cursor == d->in->len)
+			break;
+		if (pbyte_len == 0) 
+		{
+			pbyte_len = 8;
+			pbyte = d->in->buf[cursor++];
 			putc(' ', out);
-		putc(val ? ',' : '.', out);
+		}
+		putc((pbyte & 0x80) ? '1' : '0', out);
+		pbyte <<= 1;
+		--pbyte_len;
 	}
-	for (i = 0; i < count; i++, j++)
-	{
-		uint32_t val;
-		DBA_RUN_OR_RETURN(bitvec_get_bits(vec, 1, &val));
-		if (j != 0 && (j % 8) == 0)
-			putc(' ', out);
-		putc(val ? '1' : '0', out);
-	}
-	bitvec_delete(vec);
-	return dba_error_ok();
 }
-#endif
 
 
 /* Forward declaration of this static function, as we write recursive decoding
@@ -187,14 +204,14 @@ static dba_err bufr_decode_data_section(decoder d);
 		TRACE(next " starts at %d and contains at least %d bytes\n", start - d->in->buf, datalen); \
 	} while (0)
 
-dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
+dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_msg out)
 {
 	dba_err err;
 	decoder d = NULL;
 	int has_optional;
 	int i;
 
-	bufrex_raw_reset(out);
+	bufrex_msg_reset(out);
 
 	DBA_RUN_OR_RETURN(decoder_create(&d));
 	d->in = in;
@@ -214,11 +231,11 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 
 	/* Read bufr section 1 (Identification section) */
 	CHECK_AVAILABLE_DATA(d->sec1, 18, "section 1 of BUFR message (identification section)");
-	d->sec2 = d->sec1 + (ntohl(*(uint32_t*)(d->sec1)) >> 8);
+	d->sec2 = d->sec1 + readNumber(d->sec1, 3);
 	if (d->sec2 > d->in->buf + d->in->len)
 		PARSE_ERROR(d->sec1, "Section 1 claims to end past the end of the BUFR message");
 
-	has_optional = d->sec1[7] & 0x80;
+	has_optional = (d->sec1[7] & 0x80) ? 1 : 0;
 	d->out->opt.bufr.origin = d->sec1[5];
 	d->out->opt.bufr.master_table = d->sec1[10];
 	d->out->opt.bufr.local_table = d->sec1[11];
@@ -233,11 +250,14 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 	if ((int)d->sec1[17] != 0)
 		d->out->rep_year = (int)d->sec1[17] * 100 + (d->out->rep_year % 100);
 
-	TRACE(" -> category %d subcategory %d\n", 
-				(int)d->sec1[8],
-				(int)d->sec1[9]);
+	TRACE(" -> opt %d upd %d origin %d.%d tables %d.%d type %d.%d %04d-%02d-%02d %02d:%02d\n", 
+			has_optional, (int)d->sec1[6],
+			d->out->opt.bufr.origin, (int)d->sec1[4],
+			d->out->opt.bufr.master_table, d->out->opt.bufr.local_table,
+			d->out->type, d->out->subtype,
+			d->out->rep_year, d->out->rep_month, d->out->rep_day, d->out->rep_hour, d->out->rep_minute);
 
-	DBA_RUN_OR_GOTO(fail, bufrex_raw_load_tables(d->out));
+	DBA_RUN_OR_GOTO(fail, bufrex_msg_load_tables(d->out));
 
 #if 0
 	fprintf(stderr, "S1 Len %d  table #%d  osc %d  oc %d  seq #%d  optsec %x\n",
@@ -265,7 +285,7 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 	if (has_optional)
 	{
 		CHECK_AVAILABLE_DATA(d->sec2, 4, "section 2 of BUFR message (optional section)");
-		d->sec3 = d->sec2 + (ntohl(*(uint32_t*)(d->sec2)) >> 8);
+		d->sec3 = d->sec2 + readNumber(d->sec2, 3);
 		if (d->sec3 > d->in->buf + d->in->len)
 			PARSE_ERROR(d->sec2, "Section 2 claims to end past the end of the BUFR message");
 	} else
@@ -273,15 +293,18 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 
 	/* Read BUFR section 3 (Data description section) */
 	CHECK_AVAILABLE_DATA(d->sec3, 8, "section 3 of BUFR message (data description section)");
-	d->sec4 = d->sec3 + (ntohl(*(uint32_t*)(d->sec3)) >> 8);
+	d->sec4 = d->sec3 + readNumber(d->sec3, 3);
 	if (d->sec4 > d->in->buf + d->in->len)
 		PARSE_ERROR(d->sec3, "Section 3 claims to end past the end of the BUFR message");
-	d->subsets = ntohs(*(uint16_t*)(d->sec3 + 4));
+	d->subsets = readNumber(d->sec3 + 4, 2);
+	d->compression = (d->sec3[6] & 0x40) ? 1 : 0;
 	for (i = 0; i < (d->sec4 - d->sec3 - 8)/2; i++)
 	{
-		dba_varcode var = (dba_varcode)ntohs(*(uint16_t*)(d->sec3 + 7 + (i*2)));
-		DBA_RUN_OR_GOTO(fail, bufrex_raw_append_datadesc(d->out, var));
+		dba_varcode var = (dba_varcode)readNumber(d->sec3 + 7 + i * 2, 2);
+		DBA_RUN_OR_GOTO(fail, bufrex_msg_append_datadesc(d->out, var));
 	}
+	TRACE(" subsets %d observed %d compression %d byte7 %x\n",
+			d->subsets, (d->sec3[6] & 0x80) ? 1 : 0, d->compression, (unsigned int)d->sec3[6]);
 	/*
 	IFTRACE{
 		TRACE(" -> data descriptor section: ");
@@ -293,27 +316,36 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 	/* Read BUFR section 4 (Data section) */
 	CHECK_AVAILABLE_DATA(d->sec4, 4, "section 4 of BUFR message (data section)");
 
-	d->sec5 = d->sec4 + (ntohl(*(uint32_t*)(d->sec4)) >> 8);
+	d->sec5 = d->sec4 + readNumber(d->sec4, 3);
 	if (d->sec5 > d->in->buf + d->in->len)
 		PARSE_ERROR(d->sec4, "Section 4 claims to end past the end of the BUFR message");
-	TRACE("section 4 is %d bytes long (%02x %02x %02x %02x)\n", (ntohl(*(uint32_t*)(d->sec4)) >> 8),
+	TRACE("section 4 is %d bytes long (%02x %02x %02x %02x)\n", readNumber(d->sec4, 3),
 			(unsigned int)*(d->sec4), (unsigned int)*(d->sec4+1), (unsigned int)*(d->sec4+2), (unsigned int)*(d->sec4+3));
 
 	/* Initialize bit-decoding structures */
 	d->cursor = d->sec4 + 4 - d->in->buf;
 
-	/* Iterate on the number of subgroups */
-	for (i = 0; i < d->subsets; i++)
+	if (d->compression)
 	{
-		DBA_RUN_OR_GOTO(fail, bufrex_raw_get_datadesc(d->out, &(d->ops)));
+		/* Only needs to parse once */
+		d->current_subset = NULL;
+		DBA_RUN_OR_GOTO(fail, bufrex_msg_get_datadesc(d->out, &(d->ops)));
 		DBA_RUN_OR_GOTO(fail, bufr_decode_data_section(d));
+	} else {
+		/* Iterate on the number of subgroups */
+		for (i = 0; i < d->subsets; ++i)
+		{
+			DBA_RUN_OR_GOTO(fail, bufrex_msg_get_subset(d->out, i, &(d->current_subset)));
+			DBA_RUN_OR_GOTO(fail, bufrex_msg_get_datadesc(d->out, &(d->ops)));
+			DBA_RUN_OR_GOTO(fail, bufr_decode_data_section(d));
+		}
 	}
 
 	IFTRACE {
 	if (decoder_bits_left(d) > 15)
 	{
 		fprintf(stderr, "The data section of %s still contains %d unparsed bits\n",
-				d->in->file->name, decoder_bits_left(d));
+				FILENAME(d), decoder_bits_left(d));
 		/*
 		err = dba_error_parse(msg->file->name, POS + vec->cursor,
 				"the data section still contains %d unparsed bits",
@@ -329,8 +361,13 @@ dba_err bufr_decoder_decode(dba_rawmsg in, bufrex_raw out)
 	if (memcmp(d->sec5, "7777", 4) != 0)
 		PARSE_ERROR(d->sec5, "section 5 does not contain '7777'");
 
-	/* Copy the decoded attributes into the decoded variables */
-	DBA_RUN_OR_GOTO(fail, bufrex_raw_apply_attributes(out));
+	for (i = 0; i < d->subsets; ++i)
+	{
+		bufrex_subset subset;
+		DBA_RUN_OR_GOTO(fail, bufrex_msg_get_subset(out, i, &subset));
+		/* Copy the decoded attributes into the decoded variables */
+		DBA_RUN_OR_GOTO(fail, bufrex_subset_apply_attributes(subset));
+	}
 
 	if (d != NULL)
 		decoder_delete(d);
@@ -437,7 +474,9 @@ static dba_err bufr_decode_b_data(decoder d)
 		TRACE("\n");
 	}
 
-	DBA_RUN_OR_RETURN(bufrex_raw_query_btable(d->out, d->ops->val, &info));
+	DBA_RUN_OR_RETURN(dba_vartable_query_altered(
+				d->out->btable, d->ops->val,
+				DBA_ALT(d->c_width_change, d->c_scale_change), &info));
 
 	IFTRACE {
 		TRACE("Parsing @%d+%d [bl %d+%d sc %d+%d ref %d]: %d%02d%03d %s[%s]\n", d->cursor, 8-d->pbyte_len,
@@ -446,12 +485,14 @@ static dba_err bufr_decode_b_data(decoder d)
 				info->bit_ref,
 				DBA_VAR_F(info->var), DBA_VAR_X(info->var), DBA_VAR_Y(info->var),
 				info->desc, info->unit);
-		/*DBA_RUN_OR_RETURN(dump_bits(d->in->buf + d->cursor - 1, 8 - d->pbyte_len, 32, stderr));*/
+		dump_next_bits(d, 64, stderr);
 		TRACE("\n");
 	}
 
-	/* Create the new dba_var */
-	DBA_RUN_OR_GOTO(cleanup, dba_var_create(info, &var));
+	if (d->c_scale_change > 0)
+		TRACE("Applied %d scale change\n", d->c_scale_change);
+	if (d->c_width_change > 0)
+		TRACE("Applied %d width change\n", d->c_width_change);
 
 	/* Get the real datum */
 	if (info->is_string)
@@ -489,44 +530,128 @@ static dba_err bufr_decode_b_data(decoder d)
 			for (--len; len > 0 && isspace(str[len]);
 					len--)
 				str[len] = 0;
-
-			DBA_RUN_OR_GOTO(cleanup, dba_var_setc(var, str));
 		}
 
 		TRACE("bufr_message_decode_b_data len %d val %s missing %d info-len %d info-desc %s\n", len, str, missing, info->bit_len, info->desc);
+
+		/* Store the variable that we found */
+		if (d->compression)
+		{
+			uint32_t diffbits;
+			int i;
+			/* If compression is in use, then we just decoded the base value.  Now
+			 * we need to decode all the offsets */
+
+			/* Decode the number of bits (encoded in 6 bits) that these difference
+			 * values occupy */
+			DBA_RUN_OR_GOTO(cleanup, decoder_get_bits(d, 6, &diffbits));
+			if (diffbits != 0)
+			{
+				err = dba_error_unimplemented("applying difference values (this one has %d bits) on string variables in BUFR encoded with data compression", diffbits);
+				goto cleanup;
+			}
+			/* Add the string to all the subsets */
+			for (i = 0; i < d->subsets; ++i)
+			{
+				bufrex_subset subset;
+				DBA_RUN_OR_GOTO(cleanup, bufrex_msg_get_subset(d->out, i, &subset));
+
+				/* Create the new dba_var */
+				if (missing)
+					DBA_RUN_OR_GOTO(cleanup, dba_var_create(info, &var));
+				else
+					DBA_RUN_OR_GOTO(cleanup, dba_var_createc(info, str, &var));
+				DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(subset, var));
+				var = NULL;
+			}
+		} else {
+			/* Create the new dba_var */
+			if (missing)
+				DBA_RUN_OR_GOTO(cleanup, dba_var_create(info, &var));
+			else
+				DBA_RUN_OR_GOTO(cleanup, dba_var_createc(info, str, &var));
+			DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(d->current_subset, var));
+			var = NULL;
+		}
 	} else {
 		/* Read a value */
 		uint32_t val;
-
-		/* FIXME: this alters the global shared info definition! */
-		
-		/* Change the variable definition according to the current C modifiers */
-		info->scale += d->c_scale_change;
-		info->bit_len += d->c_width_change;
-		if (d->c_scale_change > 0)
-			TRACE("Applied %d scale change\n", d->c_scale_change);
-		if (d->c_width_change > 0)
-			TRACE("Applied %d scale change\n", d->c_width_change);
+		int missing;
 
 		DBA_RUN_OR_GOTO(cleanup, decoder_get_bits(d, info->bit_len, &val));
 
-		TRACE("Reading %s, size %d, starting point %d\n", info->desc, info->bit_len, val);
+		TRACE("Reading %s, size %d, scale %d, starting point %d\n", info->desc, info->bit_len, info->scale, val);
 
 		/* Check if there are bits which are not 1 (that is, if the value is present) */
-		if (val != (((1 << (info->bit_len - 1))-1) | (1 << (info->bit_len - 1))))
-		{
-			double dval = bufr_decode_int(d, val, info);
-			TRACE("Decoded as %f\n", dval);
-			DBA_RUN_OR_GOTO(cleanup, dba_var_setd(var, dval));
-		}
+		missing = (val == (((1 << (info->bit_len - 1))-1) | (1 << (info->bit_len - 1))));
 
 		/*bufr_decoder_debug(decoder, "  %s: %d%s\n", info.desc, val, info.type);*/
 		TRACE("bufr_message_decode_b_data len %d val %d info-len %d info-desc %s\n", info->bit_len, val, info->bit_len, info->desc);
-	}
 
-	/* Store the variable that we found */
-	DBA_RUN_OR_GOTO(cleanup, bufrex_raw_store_variable(d->out, var));
-	var = NULL;
+		/* Store the variable that we found */
+		if (d->compression)
+		{
+			uint32_t diffbits;
+			int i;
+			/* If compression is in use, then we just decoded the base value.  Now
+			 * we need to decode all the offsets */
+
+			/* Decode the number of bits (encoded in 6 bits) that these difference
+			 * values occupy */
+			DBA_RUN_OR_GOTO(cleanup, decoder_get_bits(d, 6, &diffbits));
+			if (missing && diffbits != 0)
+			{
+				err = dba_error_consistency("When decoding compressed BUFR data, the difference bit length must be 0 (and not %d like in this case) when the base value is missing", diffbits);
+				goto cleanup;
+			}
+			for (i = 0; i < d->subsets; ++i)
+			{
+				uint32_t diff, newval;
+				/* Access the subset we are working on */
+				bufrex_subset subset;
+				DBA_RUN_OR_GOTO(cleanup, bufrex_msg_get_subset(d->out, i, &subset));
+
+				/* Decode the difference value */
+				DBA_RUN_OR_GOTO(cleanup, decoder_get_bits(d, diffbits, &diff));
+
+				/* Check if it's all 1: in that case it's a missing value */
+				if (missing || diff == (((1 << (diffbits - 1))-1) | (1 << (diffbits - 1))))
+				{
+					/* Missing value */
+					TRACE("Decoded[%d] as missing\n", i);
+
+					/* Create the new dba_var */
+					DBA_RUN_OR_GOTO(cleanup, dba_var_create(info, &var));
+				} else {
+					/* Compute the value for this subset */
+					newval = val + diff;
+					double dval = bufr_decode_int(d, newval, info);
+					TRACE("Decoded[%d] as %f\n", i, dval);
+
+					/* Create the new dba_var */
+					DBA_RUN_OR_GOTO(cleanup, dba_var_created(info, dval, &var));
+				}
+
+				/* Add it to this subset */
+				DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(subset, var));
+				var = NULL;
+			}
+		} else {
+			if (missing)
+			{
+				/* Create the new dba_var */
+				TRACE("Decoded as missing\n");
+				DBA_RUN_OR_GOTO(cleanup, dba_var_create(info, &var));
+			} else {
+				double dval = bufr_decode_int(d, val, info);
+				TRACE("Decoded as %f\n", dval);
+				/* Create the new dba_var */
+				DBA_RUN_OR_GOTO(cleanup, dba_var_created(info, dval, &var));
+			}
+			DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(d->current_subset, var));
+			var = NULL;
+		}
+	}
 
 	/* Remove from the chain the item that we handled */
 	{
@@ -581,18 +706,30 @@ static dba_err bufr_decode_r_data(decoder d)
 		DBA_RUN_OR_GOTO(cleanup, bufrex_opcode_pop(&(d->ops), &info_op));
 
 		/* Read variable informations about the delayed replicator count */
-		DBA_RUN_OR_GOTO(cleanup, bufrex_raw_query_btable(d->out, info_op->val, &info));
+		DBA_RUN_OR_GOTO(cleanup, bufrex_msg_query_btable(d->out, info_op->val, &info));
 		
 		/* Fetch the repetition count */
 		DBA_RUN_OR_GOTO(cleanup, decoder_get_bits(d, info->bit_len, &_count));
 		count = _count;
 
 		/* Insert the repetition count among the parsed variables */
-		DBA_RUN_OR_GOTO(cleanup, dba_var_createi(info, count, &rep_var));
-		DBA_RUN_OR_GOTO(cleanup, bufrex_raw_store_variable(d->out, rep_var));
+		if (d->compression)
+		{
+			int i;
+			for (i = 0; i < d->subsets; ++i)
+			{
+				bufrex_subset subset;
+				DBA_RUN_OR_GOTO(cleanup, bufrex_msg_get_subset(d->out, i, &subset));
+				DBA_RUN_OR_GOTO(cleanup, dba_var_createi(info, count, &rep_var));
+				DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(subset, rep_var));
+			}
+		} else {
+			DBA_RUN_OR_GOTO(cleanup, dba_var_createi(info, count, &rep_var));
+			DBA_RUN_OR_GOTO(cleanup, bufrex_subset_store_variable(d->current_subset, rep_var));
+		}
 
 		bufrex_opcode_delete(&info_op);
-	/*	dba_var_delete(rep_var);   rep_var is taken in charge by bufrex_raw */
+	/*	dba_var_delete(rep_var);   rep_var is taken in charge by bufrex_msg */
 
 		TRACE("bufr_decode_r_data %d items %d times (delayed)\n", group, count);
 	} else
@@ -654,10 +791,10 @@ static dba_err bufr_decode_c_data(decoder d)
 	switch (DBA_VAR_X(code))
 	{
 		case 1:
-			d->c_width_change = DBA_VAR_Y(code) - 128;
+			d->c_width_change = DBA_VAR_Y(code) == 0 ? 0 : DBA_VAR_Y(code) - 128;
 			break;
 		case 2:
-			d->c_scale_change = DBA_VAR_Y(code) - 128;
+			d->c_scale_change = DBA_VAR_Y(code) == 0 ? 0 : DBA_VAR_Y(code) - 128;
 			break;
 		case 22:
 			if (DBA_VAR_Y(code) == 0)
@@ -665,7 +802,7 @@ static dba_err bufr_decode_c_data(decoder d)
 				DBA_RUN_OR_GOTO(cleanup, bufr_decode_r_data(d));
 			} else {
 				err = dba_error_parse(
-						d->in->file->name,
+						FILENAME(d),
 						d->sec4 + 4 + decoder_offset(d) - d->in->buf,
 						"C modifier %d%02d%03d not yet supported",
 							DBA_VAR_F(code),
@@ -679,7 +816,7 @@ static dba_err bufr_decode_c_data(decoder d)
 			{
 				DBA_RUN_OR_GOTO(cleanup, bufr_decode_r_data(d));
 			} else {
-				err = dba_error_parse(d->in->file->name, d->sec4 + 4 + decoder_offset(d) - d->in->buf,
+				err = dba_error_parse(FILENAME(d), d->sec4 + 4 + decoder_offset(d) - d->in->buf,
 						"C modifiers %d%02d%03d not yet supported",
 							DBA_VAR_F(code),
 							DBA_VAR_X(code),
@@ -688,7 +825,7 @@ static dba_err bufr_decode_c_data(decoder d)
 			}
 			break;
 		default:
-			err = dba_error_parse(d->in->file->name, d->sec4 + 4 + decoder_offset(d) - d->in->buf,
+			err = dba_error_parse(FILENAME(d), d->sec4 + 4 + decoder_offset(d) - d->in->buf,
 					"C modifiers (%d%02d%03d in this case) are not yet supported",
 						DBA_VAR_F(code),
 						DBA_VAR_X(code),
@@ -740,7 +877,7 @@ static dba_err bufr_decode_data_section(decoder d)
 				/* Pop the first opcode */
 				DBA_RUN_OR_RETURN(bufrex_opcode_pop(&(d->ops), &op));
 				
-				if ((err = bufrex_raw_query_dtable(d->out, op->val, &exp)) != DBA_OK)
+				if ((err = bufrex_msg_query_dtable(d->out, op->val, &exp)) != DBA_OK)
 				{
 					bufrex_opcode_delete(&op);
 					return err;
@@ -757,7 +894,7 @@ static dba_err bufr_decode_data_section(decoder d)
 				break;
 			}
 			default:
-				return dba_error_parse(d->in->file->name, d->sec4 + 4 + decoder_offset(d) - d->in->buf,
+				return dba_error_parse(FILENAME(d), d->sec4 + 4 + decoder_offset(d) - d->in->buf,
 						"cannot handle field %01d%02d%03d",
 							DBA_VAR_F(d->ops->val),
 							DBA_VAR_X(d->ops->val),
