@@ -22,29 +22,53 @@
 #define _GNU_SOURCE
 /* _GNU_SOURCE is defined to have asprintf */
 
-#include <dballe/init.h>
 #include <dballe/core/verbose.h>
 #include <dballe/core/aliases.h>
-#include <dballe/db/dba_db.h>
+#include <dballe/db/db.h>
 #include <dballe/db/cursor.h>
 #include <dballe/db/internals.h>
-#include <dballe/formatter.h>
+#include <dballe/msg/formatter.h>
 
 #include <f77.h>
-#include <limits.h>
 #include <float.h>
+#include <limits.h>
 
 #include <stdio.h>	// snprintf
 #include <string.h>	// strncpy
-#include <math.h>	// strncpy
+#include <math.h>
 
 #include "handles.h"
 
-#define MISSING_STRING ""
+//#define TRACEMISSING(type) fprintf(stderr, "SET TO MISSING (" type ")\n")
+#define TRACEMISSING(type) do {} while(0)
+
+/*
+ * First attempt using constants
+ */
+//#define MISSING_STRING ""
 #define MISSING_BYTE 0x7f
+// integer 2 byte   32767
 #define MISSING_INT 0x7fffffff
-#define MISSING_REAL (-1.1754944E-38)
-#define MISSING_DOUBLE (-2.22507E-308)
+//#define MISSING_REAL (3.4028235E+38f)
+#define MISSING_REAL FLT_MAX
+#define MISSING_DOUBLE   DBL_MAX
+//#define MISSING_DOUBLE   (1.79769E+308)
+//#define MISSING_DOUBLE   (1.7976931348623167E+308)
+//#define MISSING_DOUBLE   (1.797693134862316E+308)
+//#define MISSING_DOUBLE   (1.79769313486E+308)
+//#define MISSING_DOUBLE ((double)0x7FEFFFFFFFFFFFFF)
+
+/*
+ * Second attempt getting values from fortran
+extern F77_INTEGER_TYPE __missing__fortran_missing_int;
+extern F77_BYTE_TYPE __missing__fortran_missing_byte;
+extern F77_DOUBLE_TYPE __missing__fortran_missing_double;
+extern F77_REAL_TYPE __missing__fortran_missing_real;
+#define MISSING_BYTE __missing__fortran_missing_byte
+#define MISSING_INT __missing__fortran_missing_int
+#define MISSING_REAL __missing__fortran_missing_real
+#define MISSING_DOUBLE __missing__fortran_missing_double
+ */
 
 #define PERM_ANA_RO		(1 << 0)
 #define PERM_ANA_WRITE		(1 << 1)
@@ -223,7 +247,6 @@ F77_INTEGER_FUNCTION(idba_presentati)(
 	{
 		fdba_handle_init_session();
 		fdba_handle_init_simple();
-		DBA_RUN_OR_RETURN(dba_init());
 	}
 	++usage_refcount;
 
@@ -238,7 +261,6 @@ F77_INTEGER_FUNCTION(idba_presentati)(
 	return dba_error_ok();
 
 fail:
-	dba_shutdown();
 	return err;
 }
 
@@ -256,8 +278,14 @@ F77_SUBROUTINE(idba_arrivederci)(INTEGER(dbahandle))
 	FDBA_HANDLE(session, *dbahandle).session = NULL;
 	fdba_handle_release_session(*dbahandle);
 
+	/*
+	dba_shutdown does not exist anymore, but I keep this code commented out
+	here as a placeholder if in the future we'll need to hook actions when the
+	usage refcount goes to 0
+
 	if (--usage_refcount == 0)
 		dba_shutdown();
+	*/
 }
 
 static int check_flag(const char* val, const char* buf, int len)
@@ -489,52 +517,12 @@ F77_SUBROUTINE(idba_ricominciamo)(INTEGER(handle))
 }
 */
 
-#if 0
-static int get_first_id(dba_record rec)
-{
-	dba_record_cursor cur = dba_record_iterate_first(rec);
-	if (cur == NULL)
-		return 0;
-	else
-		return dba_record_cursor_id(cur);
-}
-
-static dba_varcode get_first_varcode(dba_record rec)
-{
-	dba_record_cursor cur = dba_record_iterate_first(rec);
-	if (cur == NULL)
-		return 0;
-	else
-		return dba_var_code(dba_record_cursor_variable(cur));
-}
-#endif
-
 /**@name idba_enq*
  * @anchor idba_enq
  * Functions used to read the output values of the DBALLE action routines
  * @{
  */
 
-/* TODO: find a way to pass 'parameter' to avoid a string copy */
-
-static dba_err lookup_var(dba_record rec, const char* name, dba_var* var)
-{
-	dba_varcode code = 0;
-
-	if (name[0] != 'B' && (code = dba_varcode_alias_resolve(name)) == 0)
-	{
-		dba_keyword param = dba_record_keyword_byname(name);
-		if (param == DBA_KEY_ERROR)
-			return dba_error_notfound("looking for misspelled parameter \"%s\"", name);
-		*var = dba_record_key_peek(rec, param);
-	} else {
-		if (code == 0)
-			code = DBA_STRING_TO_VAR(name + 1);
-		*var = dba_record_var_peek(rec, code);
-	}
-	return dba_error_ok();
-}
-
 /**
  * Read one integer value from the output record
  * 
@@ -561,18 +549,9 @@ F77_INTEGER_FUNCTION(idba_enqi)(
 	GENPTR_INTEGER(value)
 	char parm[20];
 	char* p;
+	int found;
 	dba_record rec;
-	dba_var var;
 
-	/*
-	else if (parameter_length >= 7 &&
-	           memcmp(parameter, "data_id ", parameter_length > 7 ? 8 : 7) == 0)
-	{
-		*value = get_first_id(STATE.output);
-		return dba_error_ok();
-	}
-	*/
-	
 	cnfImpn(parameter, parameter_length, 19, parm); parm[19] = 0;
 
 	switch (parm[0])
@@ -587,13 +566,10 @@ F77_INTEGER_FUNCTION(idba_enqi)(
 			break;
 	}
 
-	DBA_RUN_OR_RETURN(lookup_var(rec, p, &var));
-	if (var == NULL)
-	{
+	DBA_RUN_OR_RETURN(dba_record_enqi(rec, p, value, &found));
+	if (!found)
 		*value = MISSING_INT;
-		return dba_error_ok();
-	} else
-		return dba_var_enqi(var, value);
+	return dba_error_ok();
 }
 
 /**
@@ -622,8 +598,8 @@ F77_INTEGER_FUNCTION(idba_enqb)(
 	GENPTR_BYTE(value)
 	char parm[20];
 	char* p;
+	int ival, found;
 	dba_record rec;
-	dba_var var;
 
 	cnfImpn(parameter, parameter_length, 19, parm); parm[19] = 0;
 
@@ -639,19 +615,16 @@ F77_INTEGER_FUNCTION(idba_enqb)(
 			break;
 	}
 
-	DBA_RUN_OR_RETURN(lookup_var(rec, p, &var));
-	if (var == NULL)
-	{
+	DBA_RUN_OR_RETURN(dba_record_enqi(rec, p, &ival, &found));
+	if (!found)
 		*value = MISSING_BYTE;
-		return dba_error_ok();
-	} else {
-		int val;
-		dba_err res = dba_var_enqi(var, &val);
-		if (val < SCHAR_MIN || val > SCHAR_MAX)
-			return dba_error_consistency("value queried (%d) does not fit in a byte", val);
-		*value=(char)val;
-		return res;
+	else
+	{
+		if (ival < SCHAR_MIN || ival > SCHAR_MAX)
+			return dba_error_consistency("value queried (%d) does not fit in a byte", ival);
+		*value=(char)ival;
 	}
+	return dba_error_ok();
 }
 
 
@@ -681,10 +654,9 @@ F77_INTEGER_FUNCTION(idba_enqr)(
 	GENPTR_REAL(value)
 	char parm[20];
 	char* p;
-	dba_err err;
 	double dval;
+	int found;
 	dba_record rec;
-	dba_var var;
 
 	cnfImpn(parameter, parameter_length, 19, parm); parm[19] = 0;
 
@@ -697,18 +669,16 @@ F77_INTEGER_FUNCTION(idba_enqr)(
 		p = parm;
 	}
 
-	DBA_RUN_OR_RETURN(lookup_var(rec, p, &var));
-	if (var == NULL)
+	DBA_RUN_OR_RETURN(dba_record_enqd(rec, p, &dval, &found));
+	if (!found)
 	{
 		*value = MISSING_REAL;
-		return dba_error_ok();
 	} else {
-		err = dba_var_enqd(var, &dval);
 		if (dval < -FLT_MAX || dval > FLT_MAX)
 			return dba_error_consistency("value queried (%f) does not fit in a real", dval);
 		*value = dval;
-		return err;
 	}
+	return dba_error_ok();
 }
 
 /**
@@ -737,8 +707,8 @@ F77_INTEGER_FUNCTION(idba_enqd)(
 	GENPTR_DOUBLE(value)
 	char parm[20];
 	char* p;
+	int found;
 	dba_record rec;
-	dba_var var;
 
 	cnfImpn(parameter, parameter_length, 19, parm); parm[19] = 0;
 
@@ -751,13 +721,18 @@ F77_INTEGER_FUNCTION(idba_enqd)(
 		p = parm;
 	}
 
-	DBA_RUN_OR_RETURN(lookup_var(rec, p, &var));
-	if (var == NULL)
+	/*
 	{
+		double mv = MISSING_DOUBLE;
+		fprintf(stderr, "%llx\n", *(long long int*)&mv);
+		fprintf(stderr, "%.20le\n", MISSING_DOUBLE);
+	}
+	*/
+
+	DBA_RUN_OR_RETURN(dba_record_enqd(rec, p, value, &found));
+	if (!found)
 		*value = MISSING_DOUBLE;
-		return dba_error_ok();
-	} else
-		return dba_var_enqd(var, value);
+	return dba_error_ok();
 }
 
 /**
@@ -788,9 +763,7 @@ F77_INTEGER_FUNCTION(idba_enqc)(
 	char parm[20];
 	char* p;
 	dba_record rec;
-	dba_var var;
 	const char* strval;
-	dba_err err;
 
 	cnfImpn(parameter, parameter_length, 19, parm); parm[19] = 0;
 
@@ -803,17 +776,21 @@ F77_INTEGER_FUNCTION(idba_enqc)(
 		p = parm;
 	}
 
-	DBA_RUN_OR_RETURN(lookup_var(rec, p, &var));
-	if (var == NULL)
+	DBA_RUN_OR_RETURN(dba_record_enqc(rec, p, &strval));
+	if (strval == NULL)
 	{
-		cnfExprt(MISSING_STRING, value, value_length);
-		return dba_error_ok();
-	} else {
-		err = dba_var_enqc(var, &strval);
-		if (err == DBA_OK)
-			cnfExprt(strval, value, value_length);
-		return err;
+		//fprintf(stderr, "RETURN EMPTY STRING[%d]\n", value_length);
+		if (value_length > 0)
+		{
+			// The missing string value has been defined as a
+			// null byte plus blank padding.
+			value[0] = 0;
+			memset(value+1, ' ', value_length - 1);
+		}
 	}
+	else
+		cnfExprt(strval, value, value_length);
+	return dba_error_ok();
 }
 
 /*@}*/
@@ -874,7 +851,10 @@ F77_INTEGER_FUNCTION(idba_seti)(
 			return dba_error_notfound("looking for misspelled parameter \"%s\"", p);
 
 		if (*value == MISSING_INT)
+		{
+			TRACEMISSING("int");
 			return dba_record_key_unset(rec, param);
+		}
 		else
 			switch (param)
 			{
@@ -895,7 +875,10 @@ F77_INTEGER_FUNCTION(idba_seti)(
 			code = DBA_STRING_TO_VAR(p + 1);
 
 		if (*value == MISSING_INT)
+		{
+			TRACEMISSING("int");
 			return dba_record_var_unset(rec, code);
+		}
 
 		return dba_record_var_seti(rec, code, *value);
 	}
@@ -952,7 +935,10 @@ F77_INTEGER_FUNCTION(idba_setb)(
 			return dba_error_notfound("looking for misspelled parameter \"%s\"", p);
 
 		if (*value == MISSING_BYTE)
+		{
+			TRACEMISSING("byte");
 			return dba_record_key_unset(rec, param);
+		}
 		else
 			switch (param)
 			{
@@ -973,7 +959,10 @@ F77_INTEGER_FUNCTION(idba_setb)(
 			code = DBA_STRING_TO_VAR(p + 1);
 
 		if (*value == MISSING_BYTE)
+		{
+			TRACEMISSING("byte");
 			return dba_record_var_unset(rec, code);
+		}
 
 		return dba_record_var_seti(rec, code, *value);
 	}
@@ -1031,7 +1020,10 @@ F77_INTEGER_FUNCTION(idba_setr)(
 			return dba_error_notfound("looking for misspelled parameter \"%s\"", p);
 
 		if (*value == MISSING_REAL)
+		{
+			TRACEMISSING("real");
 			return dba_record_key_unset(rec, param);
+		}
 		else
 			switch (param)
 			{
@@ -1052,7 +1044,10 @@ F77_INTEGER_FUNCTION(idba_setr)(
 			code = DBA_STRING_TO_VAR(p + 1);
 
 		if (*value == MISSING_REAL)
+		{
+			TRACEMISSING("real");
 			return dba_record_var_unset(rec, code);
+		}
 
 		return dba_record_var_setd(rec, code, *value);
 	}
@@ -1102,14 +1097,29 @@ F77_INTEGER_FUNCTION(idba_setd)(
 			break;
 	}
 
+	/*
+	{
+		double mv = MISSING_DOUBLE;
+		fprintf(stderr, "MINE %llx\n", *(long long int*)&mv);
+		fprintf(stderr, "FORT %llx\n", *(long long int*)value);
+	}
+	fprintf(stderr, "IN %f\nIN %f\nComp: %d\n",
+		(double)*value,
+		(double)MISSING_DOUBLE,
+		*value == MISSING_DOUBLE);
+	*/
+
 	if (p[0] != 'B' && (code = dba_varcode_alias_resolve(p)) == 0)
 	{
 		dba_keyword param = dba_record_keyword_byname(p);
 		if (param == DBA_KEY_ERROR)
 			return dba_error_notfound("looking for misspelled parameter \"%s\"", p);
 
-		if (*value == MISSING_DOUBLE)
+		if (!isnormal(*value) || *value == MISSING_DOUBLE)
+		{
+			TRACEMISSING("double");
 			return dba_record_key_unset(rec, param);
+		}
 		else
 			switch (param)
 			{
@@ -1129,8 +1139,11 @@ F77_INTEGER_FUNCTION(idba_setd)(
 		if (code == 0)
 			code = DBA_STRING_TO_VAR(p + 1);
 
-		if (*value == MISSING_DOUBLE)
+		if (!isnormal(*value) || *value == MISSING_DOUBLE)
+		{
+			TRACEMISSING("double");
 			return dba_record_var_unset(rec, code);
+		}
 
 		return dba_record_var_setd(rec, code, *value);
 	}
@@ -1190,7 +1203,10 @@ F77_INTEGER_FUNCTION(idba_setc)(
 			return dba_error_notfound("looking for misspelled parameter \"%s\"", p);
 
 		if (val[0] == 0)
+		{
+			TRACEMISSING("char");
 			return dba_record_key_unset(rec, param);
+		}
 		else
 			switch (param)
 			{
@@ -1208,7 +1224,10 @@ F77_INTEGER_FUNCTION(idba_setc)(
 		return dba_record_key_setc(rec, param, val);
 	} else {
 		if (code == 0)
+		{
+			TRACEMISSING("char");
 			code = DBA_STRING_TO_VAR(p + 1);
+		}
 
 		if (val[0] == 0)
 			return dba_record_var_unset(rec, code);
@@ -1256,10 +1275,14 @@ F77_INTEGER_FUNCTION(idba_enqlevel)(
 	GENPTR_INTEGER(ltype)
 	GENPTR_INTEGER(l1)
 	GENPTR_INTEGER(l2)
+	int found;
 
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_LEVELTYPE, ltype));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_L1, l1));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_L2, l2));
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_LEVELTYPE, ltype, &found));
+	if (!found) *ltype = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_L1, l1, &found));
+	if (!found) *l1 = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_L2, l2, &found));
+	if (!found) *l2 = MISSING_INT;
 	return dba_error_ok();
 }
 
@@ -1318,10 +1341,14 @@ F77_INTEGER_FUNCTION(idba_enqtimerange)(
 	GENPTR_INTEGER(ptype)
 	GENPTR_INTEGER(p1)
 	GENPTR_INTEGER(p2)
+	int found;
 
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_PINDICATOR, ptype));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_P1, p1));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_P2, p2));
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_PINDICATOR, ptype, &found));
+	if (!found) *ptype = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_P1, p1, &found));
+	if (!found) *p1 = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_P2, p2, &found));
+	if (!found) *p2 = MISSING_INT;
 	return dba_error_ok();
 }
 
@@ -1392,13 +1419,20 @@ F77_INTEGER_FUNCTION(idba_enqdate)(
 	GENPTR_INTEGER(hour)
 	GENPTR_INTEGER(min)
 	GENPTR_INTEGER(sec)
+	int found;
 
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_YEAR, year));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_MONTH, month));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_DAY, day));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_HOUR, hour));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_MIN, min));
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_SEC, sec));
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_YEAR, year, &found));
+	if (!found) *year = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_MONTH, month, &found));
+	if (!found) *month = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_DAY, day, &found));
+	if (!found) *day = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_HOUR, hour, &found));
+	if (!found) *hour = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_MIN, min, &found));
+	if (!found) *min = MISSING_INT;
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.output, DBA_KEY_SEC, sec, &found));
+	if (!found) *sec = MISSING_INT;
 	return dba_error_ok();
 }
 
@@ -1693,11 +1727,11 @@ F77_INTEGER_FUNCTION(idba_elencamele)(INTEGER(handle))
 		return dba_error_consistency("idba_elencamele called without a previous idba_quantesono");
 
 	DBA_RUN_OR_RETURN(dba_db_cursor_next(STATE.ana_cur, &has_data));
+	dba_record_clear(STATE.output);
 	if (!has_data)
 	{
 		dba_db_cursor_delete(STATE.ana_cur);
 		STATE.ana_cur = NULL;
-		dba_record_clear(STATE.output);
 		return dba_error_ok();
 	} else
 		return dba_db_cursor_to_record(STATE.ana_cur, STATE.output);
@@ -1779,6 +1813,7 @@ F77_INTEGER_FUNCTION(idba_dammelo)(
 		cnfExprt("", parameter, parameter_length);
 		dba_record_clear(STATE.output);
 	} else {
+		dba_record_clear(STATE.output);
 		DBA_RUN_OR_RETURN(dba_db_cursor_to_record(STATE.query_cur, STATE.output));
 		DBA_RUN_OR_RETURN(dba_record_key_enqc(STATE.output, DBA_KEY_VAR, &varstr));
 
@@ -1899,9 +1934,12 @@ F77_INTEGER_FUNCTION(idba_dimenticami)(
 static dba_err get_referred_data_id(int* handle, int* id_context, dba_varcode* id_var)
 {
 	const char* val;
+	int found;
 
 	/* Read context ID */
-	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.qcinput, DBA_KEY_CONTEXT_ID, id_context));
+	DBA_RUN_OR_RETURN(dba_record_key_enqi(STATE.qcinput, DBA_KEY_CONTEXT_ID, id_context, &found));
+	if (!found)
+		return dba_error_notfound("looking for variable context id");
 
 	*id_var = 0;
 
@@ -1931,6 +1969,83 @@ static dba_err get_referred_data_id(int* handle, int* id_context, dba_varcode* i
 	return dba_error_ok();
 }
 
+/* Reads the list of QC values to operate on, for dba_voglioancora and dba_scusa */
+static dba_err read_qc_list(int* handle, dba_varcode** res_arr, size_t* res_len)
+{
+	dba_err err = DBA_OK;
+	dba_varcode* arr = NULL;
+	size_t arr_len = 0;
+	const char* val;
+
+	/* Retrieve the varcodes of the wanted QC values */
+	if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VAR)) != NULL)
+	{
+		/* Get only the QC values in *varlist */
+		if (*val != '*')
+		{
+			err = dba_error_consistency("QC values must start with '*'");
+			goto cleanup;
+		}
+		if ((arr = (dba_varcode*)malloc(1 * sizeof(dba_varcode))) == NULL)
+		{
+			err = dba_error_alloc("allocating the dba_varcode array to pass to dba_qc_query");
+			goto cleanup;
+		}
+		arr_len = 1;
+		arr[0] = DBA_STRING_TO_VAR(val + 2);
+	}
+	else if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VARLIST)) != NULL)
+	{
+		/* Get only the QC values in *varlist */
+		size_t pos;
+		size_t len;
+		const char* s;
+		int i;
+
+		/* Count the number of commas (and therefore of items in the
+		 * list) to decide the size of arr */
+		for (s = val, arr_len = 1; *s; ++s)
+			if (*s == ',')
+				++arr_len;
+		if ((arr = (dba_varcode*)malloc(arr_len * sizeof(dba_varcode))) == NULL)
+		{
+			err = dba_error_alloc("allocating the dba_varcode array to pass to dba_qc_query");
+			goto cleanup;
+		}
+
+		for (pos = 0, i = 0; (len = strcspn(val + pos, ",")) > 0; pos += len + 1)
+		{
+			/*
+			fprintf(stderr, "str: \"%s\", str+pos: \"%s\", str+pos+len: \"%s\"\n",
+					val, val+pos, val+pos+len);
+			*/
+			if (*(val+pos) != '*')
+			{
+				err = dba_error_consistency("QC value names must start with '*'");
+				goto cleanup;
+			}
+			arr[i++] = DBA_STRING_TO_VAR(val + pos + 2);
+			//fprintf(stderr, "GOT %s -> B%02d%03d\n", val+pos+1, DBA_VAR_X(arr[i-1]), DBA_VAR_Y(arr[i-1]));
+		}
+
+		//fprintf(stderr, "%d found\n", arr_len);
+		//for (pos = 0; pos < arr_len; ++pos)
+		//{
+			//fprintf(stderr, "%d: %02d%03d\n", pos, DBA_VAR_X(arr[pos]), DBA_VAR_Y(arr[pos]));
+		//}
+		//fprintf(stderr, "happy\n");
+	}
+
+	*res_arr = arr;
+	*res_len = arr_len;
+	arr = NULL;
+	
+cleanup:
+	if (arr != NULL)
+		free(arr);
+	return err == DBA_OK ? dba_error_ok() : err;
+}
+
 /**@name QC functions
  * @anchor qc
  * Functions used to manipulate QC data.
@@ -1946,8 +2061,8 @@ F77_INTEGER_FUNCTION(idba_voglioancora)(INTEGER(handle), INTEGER(count))
 	GENPTR_INTEGER(handle)
 	GENPTR_INTEGER(count)
 	dba_err err = DBA_OK;
-	dba_arr_varcode arr = NULL;
-	const char* val;
+	dba_varcode* arr = NULL;
+	size_t arr_len = 0;
 	int id_context;
 	dba_varcode id_var;
 
@@ -1955,42 +2070,12 @@ F77_INTEGER_FUNCTION(idba_voglioancora)(INTEGER(handle), INTEGER(count))
 	DBA_RUN_OR_RETURN(get_referred_data_id(handle, &id_context, &id_var));
 
 	/* Retrieve the varcodes of the wanted QC values */
-	if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VAR)) != NULL)
-	{
-		/* Get only the QC values in *varlist */
-		DBA_RUN_OR_RETURN(dba_arr_varcode_create(&arr));
-		if (*val != '*')
-		{
-			err = dba_error_consistency("QC values to delete must start with '*'");
-			goto cleanup;
-		}
-		DBA_RUN_OR_GOTO(cleanup, dba_arr_varcode_append(arr, DBA_STRING_TO_VAR(val + 2)));
-	}
-	else if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VARLIST)) != NULL)
-	{
-		/* Get only the QC values in *varlist */
-		size_t pos;
-		size_t len;
-		DBA_RUN_OR_RETURN(dba_arr_varcode_create(&arr));
-		for (pos = 0; (len = strcspn(val + pos, ",")) > 0; pos += len + 1)
-		{
-			/*
-			fprintf(stderr, "str: \"%s\", str+pos: \"%s\", str+pos+len: \"%s\"\n",
-					val, val+pos, val+pos+len);
-			*/
-			if (*(val+pos) != '*')
-			{
-				err = dba_error_consistency("QC values to delete must start with '*'");
-				goto cleanup;
-			}
-			DBA_RUN_OR_GOTO(cleanup, dba_arr_varcode_append(arr, DBA_STRING_TO_VAR(val + pos + 1)));
-		}
-	}
+	DBA_RUN_OR_RETURN(read_qc_list(handle, &arr, &arr_len));
 
 	/* Do QC query */
 	DBA_RUN_OR_GOTO(cleanup, dba_db_qc_query(SESSION, id_context, id_var, 
-				arr == NULL ? NULL : dba_arr_varcode_data(arr),
-				arr == NULL ? 0 : dba_arr_varcode_size(arr),
+				arr == NULL ? NULL : arr,
+				arr == NULL ? 0 : arr_len,
 				STATE.qcoutput, &(STATE.qc_count)));
 
 	STATE.qc_iter = dba_record_iterate_first(STATE.qcoutput);
@@ -2001,7 +2086,7 @@ F77_INTEGER_FUNCTION(idba_voglioancora)(INTEGER(handle), INTEGER(count))
 
 cleanup:
 	if (arr != NULL)
-		dba_arr_varcode_delete(arr);
+		free(arr);
 	return err == DBA_OK ? dba_error_ok() : err;
 }
 
@@ -2109,8 +2194,8 @@ F77_INTEGER_FUNCTION(idba_scusa)(INTEGER(handle))
 {
 	GENPTR_INTEGER(handle)
 	dba_err err = DBA_OK;
-	dba_arr_varcode arr = NULL;
-	const char* val;
+	dba_varcode* arr = NULL;
+	size_t arr_len = 0;
 	int id_context;
 	dba_varcode id_var;
 
@@ -2120,52 +2205,21 @@ F77_INTEGER_FUNCTION(idba_scusa)(INTEGER(handle))
 	
 	DBA_RUN_OR_RETURN(get_referred_data_id(handle, &id_context, &id_var));
 
-	if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VAR)) != NULL)
-	{
-		if (arr == NULL)
-			DBA_RUN_OR_RETURN(dba_arr_varcode_create(&arr));
-		if (*val != '*')
-		{
-			err = dba_error_consistency("QC values to delete must start with '*'");
-			goto cleanup;
-		}
-		DBA_RUN_OR_GOTO(cleanup, dba_arr_varcode_append(arr, DBA_STRING_TO_VAR(val + 2)));
-	}
-
-	if ((val = dba_record_key_peek_value(STATE.qcinput, DBA_KEY_VARLIST)) != NULL)
-	{
-		// Delete only the QC values in *data_id
-		size_t pos;
-		size_t len;
-		if (arr == NULL)
-			DBA_RUN_OR_RETURN(dba_arr_varcode_create(&arr));
-		for (pos = 0; (len = strcspn(val + pos, ",")) > 0; pos += len + 1)
-		{
-			/*
-			fprintf(stderr, "str: \"%s\", str+pos: \"%s\", str+pos+len: \"%s\"\n",
-					val, val+pos, val+pos+len);
-			*/
-			if (*(val+pos) != '*')
-			{
-				err = dba_error_consistency("QC values to delete must start with '*'");
-				goto cleanup;
-			}
-			DBA_RUN_OR_GOTO(cleanup, dba_arr_varcode_append(arr, DBA_STRING_TO_VAR(val + pos + 1)));
-		}
-	}
+	/* Retrieve the varcodes of the wanted QC values */
+	DBA_RUN_OR_RETURN(read_qc_list(handle, &arr, &arr_len));
 
 	// If arr is still 0, then dba_qc_delete deletes all QC values
 	DBA_RUN_OR_GOTO(cleanup,
 			dba_db_qc_remove(
 				SESSION, id_context, id_var,
-				arr == NULL ? NULL : dba_arr_varcode_data(arr),
-				arr == NULL ? 0 : dba_arr_varcode_size(arr)));
+				arr == NULL ? NULL : arr,
+				arr == NULL ? 0 : arr_len));
 
 	clear_attr_rec(STATE.qcinput);
 
 cleanup:
 	if (arr != NULL)
-		dba_arr_varcode_delete(arr);
+		free(arr);
 	return err == DBA_OK ? dba_error_ok() : err;
 }
 
@@ -2259,3 +2313,5 @@ cleanup:
 }
 
 /*@}*/
+
+/* vim:set ts=4 sw=4: */
