@@ -19,26 +19,14 @@
  * Author: Enrico Zini <enrico@enricozini.com>
  */
 
-#include <dballe/msg/dba_msg.h>
-
-#include <dballe/init.h>
-#include <dballe/dba_file.h>
-/*
-#include <dballe/bufrex/bufr_file.h>
-#include <dballe/bufrex/crex_file.h>
-#include <dballe/aof/aof_file.h>
-
-#include <dballe/bufrex/bufrex_conv.h>
-#include <dballe/aof/aof_conv.h>
-*/
-
-#include <dballe/db/dba_db.h>
+#include <dballe/msg/msg.h>
+#include <dballe/msg/file.h>
+#include <dballe/msg/bufrex_codec.h>
+#include <dballe/db/db.h>
 #include <dballe/db/import.h>
 #include <dballe/db/export.h>
-
-#include <dballe/bufrex/bufrex.h>
-
-#include <dballe/cmdline.h>
+#include <extra/cmdline.h>
+#include <extra/processor.h>
 
 #include <string.h>
 #include <stdlib.h>
@@ -47,8 +35,6 @@
 #include <sys/types.h>
 #include <pwd.h>
 
-#include "processor.h"
-
 struct grep_t grepdata = { -1, -1, -1, 0, 0, "" };
 struct poptOption grepTable[] = {
 	{ "category", 0, POPT_ARG_INT, &grepdata.category, 0,
@@ -75,6 +61,8 @@ static char* op_output_type = "bufr";
 static char* op_output_template = "";
 static int op_overwrite = 0;
 static int op_fast = 0;
+static int op_no_attrs = 0;
+static int op_full_pseudoana = 0;
 int op_verbose = 0;
 
 struct poptOption dbTable[] = {
@@ -104,22 +92,37 @@ struct import_data
 	int forced_repcod;
 };
 
-static dba_err import_message(dba_rawmsg rmsg, bufrex_raw braw, dba_msg msg, void* data)
+static dba_err import_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs, void* data)
 {
 	struct import_data* d = (struct import_data*)data;
-	if (msg == NULL)
+	int i, import_flags = 0;;
+	if (msgs == NULL)
 	{
 		fprintf(stderr, "Message #%d cannot be parsed: ignored\n",
 				rmsg->index);
+		return dba_error_ok();
 	}
-	else if (d->forced_repcod == -1 && msg->type == MSG_GENERIC)
+	if (d->overwrite)
+		import_flags |= DBA_IMPORT_OVERWRITE;
+	if (op_fast)
+		import_flags |= DBA_IMPORT_NO_TRANSACTIONS;
+	if (!op_no_attrs)
+		import_flags |= DBA_IMPORT_ATTRS;
+	if (op_full_pseudoana)
+		import_flags |= DBA_IMPORT_FULL_PSEUDOANA;
+
+	for (i = 0; i < msgs->len; ++i)
 	{
-		/* Put generic messages in the generic rep_cod by default */
-		DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, 255, d->overwrite, op_fast));
-	}
-	else
-	{
-		DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, d->forced_repcod, d->overwrite, op_fast));
+		dba_msg msg = msgs->msgs[i];
+		if (d->forced_repcod == -1 && msg->type == MSG_GENERIC)
+		{
+			/* Put generic messages in the generic rep_cod by default */
+			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, 255, import_flags));
+		}
+		else
+		{
+			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, d->forced_repcod, import_flags));
+		}
 	}
 	return dba_error_ok();
 }
@@ -139,7 +142,6 @@ dba_err do_dump(poptContext optCon)
 	DBA_RUN_OR_RETURN(dba_record_create(&query));
 	DBA_RUN_OR_RETURN(dba_cmdline_get_query(optCon, query));
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 	DBA_RUN_OR_RETURN(dba_db_query(db, query, &cursor, &count));
 	DBA_RUN_OR_RETURN(dba_record_create(&result));
@@ -156,8 +158,10 @@ dba_err do_dump(poptContext optCon)
 		dba_record_print(result, stdout);
 	}
 
+	//fprintf(stdout, "***\n");
+	//dba_record_print(query, stdout);
+
 	dba_db_delete(db);
-	dba_shutdown();
 
 	dba_record_delete(result);
 	dba_record_delete(query);
@@ -180,7 +184,6 @@ dba_err do_stations(poptContext optCon)
 	DBA_RUN_OR_RETURN(dba_record_create(&query));
 	DBA_RUN_OR_RETURN(dba_cmdline_get_query(optCon, query));
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 	DBA_RUN_OR_RETURN(dba_db_ana_query(db, query, &cursor, &count));
 	DBA_RUN_OR_RETURN(dba_record_create(&result));
@@ -198,7 +201,6 @@ dba_err do_stations(poptContext optCon)
 	}
 
 	dba_db_delete(db);
-	dba_shutdown();
 
 	dba_record_delete(result);
 	dba_record_delete(query);
@@ -218,11 +220,9 @@ dba_err do_wipe(poptContext optCon)
 	/* Get the optional name of the repinfo file */
 	table = poptGetArg(optCon);
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 	DBA_RUN_OR_RETURN(dba_db_reset(db, table));
 	dba_db_delete(db);
-	dba_shutdown();
 
 	return dba_error_ok();
 }
@@ -239,11 +239,9 @@ dba_err do_cleanup(poptContext optCon)
 	/* Get the optional name of the repinfo file */
 	table = poptGetArg(optCon);
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 	DBA_RUN_OR_RETURN(dba_db_remove_orphans(db));
 	dba_db_delete(db);
-	dba_shutdown();
 
 	return dba_error_ok();
 }
@@ -261,12 +259,10 @@ dba_err do_repinfo(poptContext optCon)
 	/* Get the optional name of the */
 	table = poptGetArg(optCon);
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 	DBA_RUN_OR_RETURN(dba_db_update_repinfo(db, table, &added, &deleted, &updated));
 	printf("Update completed: %d added, %d deleted, %d updated.\n", added, deleted, updated);
 	dba_db_delete(db);
-	dba_shutdown();
 
 	return dba_error_ok();
 }
@@ -311,7 +307,6 @@ dba_err do_import(poptContext optCon)
 
 	type = dba_cmdline_stringToMsgType(op_input_type, optCon);
 
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&data.db));
 	data.overwrite = op_overwrite;
 	DBA_RUN_OR_RETURN(parse_op_report(data.db, &(data.forced_repcod)));
@@ -319,7 +314,6 @@ dba_err do_import(poptContext optCon)
 	DBA_RUN_OR_RETURN(process_all(optCon, type, &grepdata, import_message, (void*)&data));
 
 	dba_db_delete(data.db);
-	dba_shutdown();
 
 	return dba_error_ok();
 }
@@ -332,14 +326,18 @@ struct export_data
 	int forced_rep_cod;
 };
 
-static dba_err msg_writer(dba_msg msg, void* data)
+static dba_err msg_writer(dba_msgs msgs, void* data)
 {
 	struct export_data* d = (struct export_data*)data;
 	/* Override the message type if the user asks for it */
 	if (d->forced_rep_cod != -1)
-		msg->type = dba_msg_type_from_repcod(d->forced_rep_cod);
-	DBA_RUN_OR_RETURN(dba_file_write(d->file, msg, d->cat, d->subcat));
-	dba_msg_delete(msg);
+	{
+		int i;
+		for (i = 0; i < msgs->len; ++i)
+			msgs->msgs[i]->type = dba_msg_type_from_repcod(d->forced_rep_cod);
+	}
+	DBA_RUN_OR_RETURN(dba_file_write_msgs(d->file, msgs, d->cat, d->subcat));
+	dba_msgs_delete(msgs);
 	return dba_error_ok();
 }
 
@@ -358,7 +356,6 @@ dba_err do_export(poptContext optCon)
 			dba_cmdline_error(optCon, "output template must be specified as 'type.subtype' (type number, then dot, then subtype number)");
 
 	/* Connect to the database */
-	DBA_RUN_OR_RETURN(dba_init());
 	DBA_RUN_OR_RETURN(create_dba_db(&db));
 
 	/* Create the query */
@@ -370,19 +367,48 @@ dba_err do_export(poptContext optCon)
 	DBA_RUN_OR_RETURN(parse_op_report(db, &(d.forced_rep_cod)));
 
 	type = dba_cmdline_stringToMsgType(op_output_type, optCon);
-	DBA_RUN_OR_RETURN(dba_file_create(&d.file, type, "(stdout)", "w"));
+	DBA_RUN_OR_RETURN(dba_file_create(type, "(stdout)", "w", &d.file));
 
 	DBA_RUN_OR_RETURN(dba_db_export(db, query, msg_writer, &d));
 
 	dba_file_delete(d.file);
 	dba_db_delete(db);
-	dba_shutdown();
 
 	dba_record_delete(query);
 
 	return dba_error_ok();
 }
 
+dba_err do_delete(poptContext optCon)
+{
+	dba_record query;
+	dba_db db;
+
+	/* Throw away the command name */
+	poptGetArg(optCon);
+
+	if (poptPeekArg(optCon) == NULL)
+		dba_cmdline_error(optCon, "you need to specify some query parameters");
+
+	/* Connect to the database */
+	DBA_RUN_OR_RETURN(create_dba_db(&db));
+
+	/* Create the query */
+	DBA_RUN_OR_RETURN(dba_record_create(&query));
+
+	/* Add the query data from commandline */
+	DBA_RUN_OR_RETURN(dba_cmdline_get_query(optCon, query));
+
+	// TODO: check that there is something
+
+	DBA_RUN_OR_RETURN(dba_db_remove(db, query));
+
+	dba_db_delete(db);
+	dba_record_delete(query);
+
+	return dba_error_ok();
+}
+
 static struct tool_desc dbadb;
 
 struct poptOption dbadb_dump_options[] = {
@@ -411,8 +437,11 @@ struct poptOption dbadb_import_options[] = {
 	{ "report", 'r', POPT_ARG_STRING, &op_report, 0,
 		"force data to be of this type of report, specified with rep_cod or rep_memo values", "rep" },
 	{ "fast", 0, POPT_ARG_NONE, &op_fast, 0,
-		"make import faster, but an interruption will mean that there can be "
-		"data in the database for only part of an imported message" },
+		"Ignored.  This option is left here for compatibility with old versions of dbadb." },
+	{ "no-attrs", 0, POPT_ARG_NONE, &op_no_attrs, 0,
+		"do not import data attributes" },
+	{ "full-pseudoana", 0, POPT_ARG_NONE, &op_full_pseudoana, 0,
+		"merge pseudoana extra values with the ones already existing in the database" },
 	{ NULL, 0, POPT_ARG_INCLUDE_TABLE, &dbTable, 0,
 		"Options used to connect to the database" },
 	{ NULL, 0, POPT_ARG_INCLUDE_TABLE, &grepTable, 0,
@@ -456,13 +485,20 @@ struct poptOption dbadb_stations_options[] = {
 	POPT_TABLEEND
 };
 
+struct poptOption dbadb_delete_options[] = {
+	{ "help", '?', 0, 0, 1, "print an help message" },
+	{ NULL, 0, POPT_ARG_INCLUDE_TABLE, &dbTable, 0,
+		"Options used to connect to the database" },
+	POPT_TABLEEND
+};
+
 static void init()
 {
 	dbadb.desc = "Manage the DB-ALLe database";
 	dbadb.longdesc =
 		"It allows to initialise the database, dump its contents and import and export data "
 		"using BUFR, CREX or AOF encoding";
-	dbadb.ops = (struct op_dispatch_table*)calloc(8, sizeof(struct op_dispatch_table));
+	dbadb.ops = (struct op_dispatch_table*)calloc(9, sizeof(struct op_dispatch_table));
 
 	dbadb.ops[0].func = do_dump;
 	dbadb.ops[0].aliases[0] = "dump";
@@ -530,11 +566,21 @@ static void init()
 		"complete list.";
 	dbadb.ops[6].optable = dbadb_stations_options;
 
-	dbadb.ops[7].func = NULL;
-	dbadb.ops[7].usage = NULL;
-	dbadb.ops[7].desc = NULL;
-	dbadb.ops[7].longdesc = NULL;
-	dbadb.ops[7].optable = NULL;
+	dbadb.ops[7].func = do_delete;
+	dbadb.ops[7].aliases[0] = "delete";
+	dbadb.ops[7].usage = "delete [options] [queryparm1=val1 [queryparm2=val2 [...]]]";
+	dbadb.ops[7].desc = "Delete all the data matching the given query parameters";
+	dbadb.ops[7].longdesc = "Query parameters are the same of the Fortran API. "
+		"Please see the section \"Input and output parameters -- For data "
+		"related action routines\" of the Fortran API documentation for a "
+		"complete list.";
+	dbadb.ops[7].optable = dbadb_delete_options;
+
+	dbadb.ops[8].func = NULL;
+	dbadb.ops[8].usage = NULL;
+	dbadb.ops[8].desc = NULL;
+	dbadb.ops[8].longdesc = NULL;
+	dbadb.ops[8].optable = NULL;
 };
 
 int main (int argc, const char* argv[])
